@@ -10,6 +10,10 @@
 
 namespace Botan {
 
+void Buffered_Computation::update(std::string_view str) {
+   add_data({cast_char_ptr_to_uint8(str.data()), str.size()});
+}
+
 void Buffered_Computation::update_be(uint16_t val) {
    uint8_t inb[sizeof(val)];
    store_be(val, inb);
@@ -46,4 +50,9 @@ void Buffered_Computation::update_le(uint64_t val) {
    add_data({inb, sizeof(inb)});
 }
 
+void Buffered_Computation::final(std::span<uint8_t> out) {
+   BOTAN_ARG_CHECK(out.size() >= output_length(), "provided output buffer has insufficient capacity");
+   final_result(out);
+}
+
 }  // namespace Botan
