@@ -10,6 +10,7 @@
 
 #include <botan/internal/aes.h>
 
+#include <botan/internal/isa_extn.h>
 #include <botan/internal/loadstor.h>
 #include <arm_neon.h>
 
@@ -17,48 +18,48 @@ namespace Botan {
 
 namespace AES_AARCH64 {
 
-BOTAN_FUNC_ISA_INLINE("+crypto+aes") void enc(uint8x16_t& B, uint8x16_t K) {
+BOTAN_FORCE_INLINE BOTAN_FN_ISA_AES void enc(uint8x16_t& B, uint8x16_t K) {
    B = vaesmcq_u8(vaeseq_u8(B, K));
 }
 
-BOTAN_FUNC_ISA_INLINE("+crypto+aes")
-void enc4(uint8x16_t& B0, uint8x16_t& B1, uint8x16_t& B2, uint8x16_t& B3, uint8x16_t K) {
+BOTAN_FORCE_INLINE BOTAN_FN_ISA_AES void enc4(
+   uint8x16_t& B0, uint8x16_t& B1, uint8x16_t& B2, uint8x16_t& B3, uint8x16_t K) {
    B0 = vaesmcq_u8(vaeseq_u8(B0, K));
    B1 = vaesmcq_u8(vaeseq_u8(B1, K));
    B2 = vaesmcq_u8(vaeseq_u8(B2, K));
    B3 = vaesmcq_u8(vaeseq_u8(B3, K));
 }
 
-BOTAN_FUNC_ISA_INLINE("+crypto+aes") void enc_last(uint8x16_t& B, uint8x16_t K, uint8x16_t K2) {
+BOTAN_FORCE_INLINE BOTAN_FN_ISA_AES void enc_last(uint8x16_t& B, uint8x16_t K, uint8x16_t K2) {
    B = veorq_u8(vaeseq_u8(B, K), K2);
 }
 
-BOTAN_FUNC_ISA_INLINE("+crypto+aes")
-void enc4_last(uint8x16_t& B0, uint8x16_t& B1, uint8x16_t& B2, uint8x16_t& B3, uint8x16_t K, uint8x16_t K2) {
+BOTAN_FORCE_INLINE BOTAN_FN_ISA_AES void enc4_last(
+   uint8x16_t& B0, uint8x16_t& B1, uint8x16_t& B2, uint8x16_t& B3, uint8x16_t K, uint8x16_t K2) {
    B0 = veorq_u8(vaeseq_u8(B0, K), K2);
    B1 = veorq_u8(vaeseq_u8(B1, K), K2);
    B2 = veorq_u8(vaeseq_u8(B2, K), K2);
    B3 = veorq_u8(vaeseq_u8(B3, K), K2);
 }
 
-BOTAN_FUNC_ISA_INLINE("+crypto+aes") void dec(uint8x16_t& B, uint8x16_t K) {
+BOTAN_FORCE_INLINE BOTAN_FN_ISA_AES void dec(uint8x16_t& B, uint8x16_t K) {
    B = vaesimcq_u8(vaesdq_u8(B, K));
 }
 
-BOTAN_FUNC_ISA_INLINE("+crypto+aes")
-void dec4(uint8x16_t& B0, uint8x16_t& B1, uint8x16_t& B2, uint8x16_t& B3, uint8x16_t K) {
+BOTAN_FORCE_INLINE BOTAN_FN_ISA_AES void dec4(
+   uint8x16_t& B0, uint8x16_t& B1, uint8x16_t& B2, uint8x16_t& B3, uint8x16_t K) {
    B0 = vaesimcq_u8(vaesdq_u8(B0, K));
    B1 = vaesimcq_u8(vaesdq_u8(B1, K));
    B2 = vaesimcq_u8(vaesdq_u8(B2, K));
    B3 = vaesimcq_u8(vaesdq_u8(B3, K));
 }
 
-BOTAN_FUNC_ISA_INLINE("+crypto+aes") void dec_last(uint8x16_t& B, uint8x16_t K, uint8x16_t K2) {
+BOTAN_FORCE_INLINE BOTAN_FN_ISA_AES void dec_last(uint8x16_t& B, uint8x16_t K, uint8x16_t K2) {
    B = veorq_u8(vaesdq_u8(B, K), K2);
 }
 
-BOTAN_FUNC_ISA_INLINE("+crypto+aes")
-void dec4_last(uint8x16_t& B0, uint8x16_t& B1, uint8x16_t& B2, uint8x16_t& B3, uint8x16_t K, uint8x16_t K2) {
+BOTAN_FORCE_INLINE BOTAN_FN_ISA_AES void dec4_last(
+   uint8x16_t& B0, uint8x16_t& B1, uint8x16_t& B2, uint8x16_t& B3, uint8x16_t K, uint8x16_t K2) {
    B0 = veorq_u8(vaesdq_u8(B0, K), K2);
    B1 = veorq_u8(vaesdq_u8(B1, K), K2);
    B2 = veorq_u8(vaesdq_u8(B2, K), K2);
@@ -70,7 +71,7 @@ void dec4_last(uint8x16_t& B0, uint8x16_t& B1, uint8x16_t& B2, uint8x16_t& B3, u
 /*
 * AES-128 Encryption
 */
-BOTAN_FUNC_ISA("+crypto+aes") void AES_128::hw_aes_encrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
+BOTAN_FN_ISA_AES void AES_128::hw_aes_encrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
    const uint8_t* skey = reinterpret_cast<const uint8_t*>(m_EK.data());
 
    const uint8x16_t K0 = vld1q_u8(skey + 0 * 16);
@@ -133,7 +134,7 @@ BOTAN_FUNC_ISA("+crypto+aes") void AES_128::hw_aes_encrypt_n(const uint8_t in[],
 /*
 * AES-128 Decryption
 */
-BOTAN_FUNC_ISA("+crypto+aes") void AES_128::hw_aes_decrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
+BOTAN_FN_ISA_AES void AES_128::hw_aes_decrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
    const uint8_t* skey = reinterpret_cast<const uint8_t*>(m_DK.data());
 
    const uint8x16_t K0 = vld1q_u8(skey + 0 * 16);
@@ -196,7 +197,7 @@ BOTAN_FUNC_ISA("+crypto+aes") void AES_128::hw_aes_decrypt_n(const uint8_t in[],
 /*
 * AES-192 Encryption
 */
-BOTAN_FUNC_ISA("+crypto+aes") void AES_192::hw_aes_encrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
+BOTAN_FN_ISA_AES void AES_192::hw_aes_encrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
    const uint8_t* skey = reinterpret_cast<const uint8_t*>(m_EK.data());
 
    const uint8x16_t K0 = vld1q_u8(skey + 0 * 16);
@@ -265,7 +266,7 @@ BOTAN_FUNC_ISA("+crypto+aes") void AES_192::hw_aes_encrypt_n(const uint8_t in[],
 /*
 * AES-192 Decryption
 */
-BOTAN_FUNC_ISA("+crypto+aes") void AES_192::hw_aes_decrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
+BOTAN_FN_ISA_AES void AES_192::hw_aes_decrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
    const uint8_t* skey = reinterpret_cast<const uint8_t*>(m_DK.data());
 
    const uint8x16_t K0 = vld1q_u8(skey + 0 * 16);
@@ -334,7 +335,7 @@ BOTAN_FUNC_ISA("+crypto+aes") void AES_192::hw_aes_decrypt_n(const uint8_t in[],
 /*
 * AES-256 Encryption
 */
-BOTAN_FUNC_ISA("+crypto+aes") void AES_256::hw_aes_encrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
+BOTAN_FN_ISA_AES void AES_256::hw_aes_encrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
    const uint8_t* skey = reinterpret_cast<const uint8_t*>(m_EK.data());
 
    const uint8x16_t K0 = vld1q_u8(skey + 0 * 16);
@@ -411,7 +412,7 @@ BOTAN_FUNC_ISA("+crypto+aes") void AES_256::hw_aes_encrypt_n(const uint8_t in[],
 /*
 * AES-256 Decryption
 */
-BOTAN_FUNC_ISA("+crypto+aes") void AES_256::hw_aes_decrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
+BOTAN_FN_ISA_AES void AES_256::hw_aes_decrypt_n(const uint8_t in[], uint8_t out[], size_t blocks) const {
    const uint8_t* skey = reinterpret_cast<const uint8_t*>(m_DK.data());
 
    const uint8x16_t K0 = vld1q_u8(skey + 0 * 16);
