@@ -17,7 +17,7 @@ namespace Botan {
 * Write data from a pipe into an ostream
 */
 std::ostream& operator<<(std::ostream& stream, Pipe& pipe) {
-   secure_vector<uint8_t> buffer(BOTAN_DEFAULT_BUFFER_SIZE);
+   secure_vector<uint8_t> buffer(DefaultBufferSize);
    while(stream.good() && pipe.remaining()) {
       const size_t got = pipe.read(buffer.data(), buffer.size());
       stream.write(cast_uint8_ptr_to_char(buffer.data()), got);
@@ -32,7 +32,7 @@ std::ostream& operator<<(std::ostream& stream, Pipe& pipe) {
 * Read data from an istream into a pipe
 */
 std::istream& operator>>(std::istream& stream, Pipe& pipe) {
-   secure_vector<uint8_t> buffer(BOTAN_DEFAULT_BUFFER_SIZE);
+   secure_vector<uint8_t> buffer(DefaultBufferSize);
    while(stream.good()) {
       stream.read(cast_uint8_ptr_to_char(buffer.data()), buffer.size());
       const size_t got = static_cast<size_t>(stream.gcount());
