@@ -12,6 +12,7 @@
 #include <botan/exceptn.h>
 #include <botan/secmem.h>
 #include <botan/sym_algo.h>
+#include <memory>
 #include <span>
 #include <string>
 #include <string_view>
@@ -146,7 +147,6 @@ class BOTAN_PUBLIC_API(2, 0) Cipher_Mode : public SymmetricAlgorithm {
       */
       template <concepts::resizable_byte_buffer T>
       void update(T& buffer, size_t offset = 0) {
-         BOTAN_ASSERT(buffer.size() >= offset, "Offset ok");
          const size_t written = process(std::span(buffer).subspan(offset));
          buffer.resize(offset + written);
       }
